/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 03-04-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package pl.edu.icm.unicore.uvos.db;

import java.sql.CallableStatement;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import org.apache.ibatis.type.IntegerTypeHandler;
import org.apache.ibatis.type.JdbcType;


/**
 * Extends standard MyBatis {@link IntegerTypeHandler} so -1 int parameters are
 * mapped to a SQL value "NULL". Also NULL results are mapped back to -1.
 * 
 * @author K. Benedyczak
 */
public class NullIntTypeHandler extends IntegerTypeHandler
{
	@Override
	public void setParameter(PreparedStatement ps, int i, Integer parameter,
			JdbcType jdbcType) throws SQLException
	{
		if (parameter != null && parameter instanceof Integer && 
				parameter.equals(Integer.valueOf(-1)))
			parameter = null;
		super.setParameter(ps, i, parameter, jdbcType);
	}
	
	@Override
	public Integer getResult(ResultSet rs, String columnName)
			throws SQLException 
	{
		Integer result = getNullableResult(rs, columnName);
		if (rs.wasNull()) {
			return Integer.valueOf(-1);
		} else {
			return result;
		}
	}

	@Override
	public Integer getResult(CallableStatement cs, int columnIndex)
			throws SQLException 
	{
		Integer result = getNullableResult(cs, columnIndex);
		if (cs.wasNull()) {
			return Integer.valueOf(-1);
		} else {
			return result;
		}
	}
}
