package pl.edu.icm.unicore.uvos.saml;

import org.apache.log4j.Logger;

import pl.edu.icm.unicore.uvos.util.ConfDefaults;
import pl.edu.icm.unicore.uvos.util.Configuration;

/**
 *
 * @author K. Stasiak
 */
public class SAMLConfiguration {
	private static final Logger log = Logger.getLogger(SAMLConfiguration.class);
	private boolean signRespNever;
	private boolean signRespAlways;
	private boolean signIfRespUnsigned;
	private boolean signAssertNever;
	private boolean signAssertAlways;

	private void init(Configuration conf)
	{
		String repPolicy = conf.getAsString(ConfDefaults.SIGN_RESPONSE,
				ConfDefaults.signResponse, log);
		signRespAlways = signRespNever = false;
		if (repPolicy.equals("always"))
			signRespAlways = true;
		else if (repPolicy.equals("never"))
			signRespNever = true;
		else if (!repPolicy.equals("asRequest"))
		{
			log.warn("Unknown value of " + ConfDefaults.SIGN_RESPONSE +
					", using defalut " + ConfDefaults.signResponse);
		}
		String assertionPolicy = conf.getAsString(ConfDefaults.SIGN_ASSERTION,
				ConfDefaults.signAssertion, log);
		signAssertAlways = signAssertNever = signIfRespUnsigned = false;
		if (assertionPolicy.equals("always"))
			signAssertAlways = true;
		else if (assertionPolicy.equals("never"))
			signAssertNever = true;
		else if (assertionPolicy.equals("ifResponseUnsigned"))
			signIfRespUnsigned = true;
		else if (!assertionPolicy.equals("asRequest"))
		{
			log.warn("Unknown value of " + ConfDefaults.SIGN_ASSERTION +
					", using defalut " + ConfDefaults.signAssertion);
		}
	}

	public SAMLConfiguration(Configuration conf){
		init(conf);
	}

	public boolean isSignRespNever() {
		return signRespNever;
	}

	public boolean isSignRespAlways() {
		return signRespAlways;
	}

	public boolean isSignIfRespUnsigned() {
		return signIfRespUnsigned;
	}

	public boolean isSignAssertNever() {
		return signAssertNever;
	}

	public boolean isSignAssertAlways() {
		return signAssertAlways;
	}
}
