/*
 * Copyright (c) 2011 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 02-09-2011
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */
package pl.edu.icm.unicore.uvos.server;

import java.io.File;
import java.io.IOException;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;

import pl.edu.icm.unicore.uvos.util.ConfDefaults;
import pl.edu.icm.unicore.uvos.util.Configuration;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.CISInfoProvider;
import pl.edu.icm.unicore.uvos.wsapi.xmlbeans.UVOSFault;
import xmlbeans.pl.edu.icm.uvos.cisprovider.GetServiceInfoRequestDocument;
import xmlbeans.pl.edu.icm.uvos.cisprovider.GetServiceInfoResponseDocument;
import xmlbeans.pl.edu.icm.uvos.glue2.EndpointHealthStateT;
import xmlbeans.pl.edu.icm.uvos.glue2.EndpointT;
import xmlbeans.pl.edu.icm.uvos.glue2.QualityLevelT;
import xmlbeans.pl.edu.icm.uvos.glue2.ServiceT;
import xmlbeans.pl.edu.icm.uvos.glue2.ServingStateT;

/**
 * Implementation of the GLUE 2 query. Currently only providing a very basic information
 * including service version.
 * @author K. Benedyczak
 */
public class WSGLUEInfoProviderImpl implements CISInfoProvider
{
	private static final Logger log = Logger.getLogger(JettyServer.class);
	private String baseURL;
	public static final String UVOS_HUMAN_NAME = "UNICORE Virtual Organisations Service (UVOS)";
	public static final String EMI_VERSION_FILE = "/etc/emi-version";
	
	public WSGLUEInfoProviderImpl(Configuration conf)
	{
		boolean ssl = conf.getAsBool(ConfDefaults.HTTPS_ENABLE, 
				ConfDefaults.httpsEnable, log);
		if (ssl) 
		{
			baseURL = "https://" + conf.getAsString(ConfDefaults.HTTPS_HOST, 
					ConfDefaults.httpsHost, log) + ":" +
					conf.getAsInt(ConfDefaults.HTTPS_PORT, 
						ConfDefaults.httpsPort, log);
		} else
		{
			baseURL = "http://" + conf.getAsString(ConfDefaults.HTTP_HOST, 
					ConfDefaults.httpHost, log) + ":" +
					conf.getAsInt(ConfDefaults.HTTP_PORT, 
						ConfDefaults.httpPort, log);
		}
	}
	
	@Override
	public GetServiceInfoResponseDocument getServiceInfo(
			GetServiceInfoRequestDocument req) throws UVOSFault
	{
		GetServiceInfoResponseDocument res = GetServiceInfoResponseDocument.Factory.newInstance();
		ServiceT[] arrServiceT = new ServiceT[] {getServiceDesc()};
		res.addNewGetServiceInfoResponse().setServiceArray(arrServiceT);
		return res;
	}
	
	private ServiceT getServiceDesc()
	{
		ServiceT st = ServiceT.Factory.newInstance();
		st.setID(baseURL);
		st.setName(UVOS_HUMAN_NAME);
		st.setQualityLevel(QualityLevelT.PRODUCTION);
		st.setType("unicore6.UVOSAssertionQueryService");
		st.setEndpointArray(new EndpointT[] {getServiceendpointDesc()});
		return st;
	}
	
	private EndpointT getServiceendpointDesc()
	{
		EndpointT et = EndpointT.Factory.newInstance();
		et.setID(baseURL);
		et.setURL(baseURL);
		et.setQualityLevel(QualityLevelT.PRODUCTION);
		et.setHealthState(EndpointHealthStateT.OK);
		et.setServingState(ServingStateT.PRODUCTION);
		
		String uVersion=getClass().getPackage().getSpecificationVersion();
		if (uVersion == null)
			uVersion = "DEVELOPMENT";
		et.addInterfaceVersion(uVersion);
		et.setImplementationName("UVOS");
		et.setImplementationVersion(uVersion);
		et.setInterfaceName("UVOS");

		//add EMI Version, if present
		File emiFile = new File(EMI_VERSION_FILE);
		if (emiFile.exists() && emiFile.canRead())
		{
			try 
			{
				String emiVersion = FileUtils.readFileToString(emiFile);
				et.addOtherInfo("MiddlewareName=EMI");
				et.addOtherInfo("MiddlewareVersion=" + emiVersion);
			} catch (IOException ex)
			{
				log.warn("Can't read emi version.", ex);
			}
		}
		return et;
	}
}
