/*
 * Copyright (c) 2010 ICM Uniwersytet Warszawski All rights reserved.
 * See LICENCE file for licencing information.
 *
 * Created on 2010-07-04
 * Author: K. Benedyczak <golbi@mat.umk.pl>
 */

package pl.edu.icm.unicore.uvos.tests.functional;

import static junit.framework.Assert.*;

import org.junit.Test;

import pl.edu.icm.unicore.uvos.server.CryptoDataProviderImpl;
import pl.edu.icm.unicore.uvos.util.Configuration;

/**
 * Tests {@link CryptoDataProviderImpl} class.
 * @author golbi
 */
public class CryptoDataProviderTest
{
	private Configuration getConfig(String file)
	{
		try
		{
			return new Configuration(file);
		} catch (Exception e)
		{
			e.printStackTrace();
			fail("Can't read configuration: " + e.toString());
			return null;
		}
	}
	
	@Test
	public void test1()
	{
		Configuration cnf = getConfig("src/test/resources/uvosServer-wrongAlias.conf");
		try
		{
			new CryptoDataProviderImpl(cnf);
		} catch (Exception e)
		{
			assertTrue("Wrong exception received", 
				e.getMessage().contains("doesn't contain alias"));
			return;
		}
		fail("Managed to init CryptoDataProviderImpl with non existing alias.");
	}

	@Test
	public void test2()
	{
		Configuration cnf = getConfig("src/test/resources/uvosServer-wrongAlias2.conf");
		try
		{
			new CryptoDataProviderImpl(cnf);
		} catch (Exception e)
		{
			assertTrue("Wrong exception received",
				e.getMessage().contains("is trusted certificate entry"));
			return;
		}
		fail("Managed to init CryptoDataProviderImpl with certificate alias as a key alias.");
	}
	
	@Test
	public void testOK()
	{
		Configuration cnf = getConfig("src/test/resources/uvosServer.conf");
		try
		{
			new CryptoDataProviderImpl(cnf);
		} catch (Exception e)
		{
			fail("Got error on good data");
			return;
		}
	}
	
}
